﻿using Borepin.ViewModel;
using System.Collections.Generic;
using System.Threading.Tasks;
using System.Windows.Input;
using Prism.Commands;
using Prism.Navigation;
using Borepin.Base;
using FabAccessAPI.Schema;
using Prism.Services;
using System;
using NaturalSort.Extension;
using System.Linq;
using Borepin.Service;
using static FabAccessAPI.Schema.Machine;

namespace Borepin.PageModel
{
    public class MachineListPageModel : ConnectionModelBase
    {
        #region Constructors
        public MachineListPageModel(INavigationService navigationService, IPageDialogService pageDialogService, IAPIService apiService) : base(navigationService, pageDialogService, apiService)
        {
            ScanCodeCommand = new DelegateCommand(async () => await ScanCodeCommandExecute().ConfigureAwait(false));
            RefreshCommand = new DelegateCommand(async ()=> await RefreshCommandExecute().ConfigureAwait(true));
        }
        #endregion

        #region Data
        public override async Task LoadAPIData()
        {
            User user_self = await _API.Session.UserSystem.Info.GetUserSelf().ConfigureAwait(false);

            IReadOnlyList<Machine> machine_list = await _API.Session.MachineSystem.Info.GetMachineList().ConfigureAwait(false);

            List<MachineListItemViewModel> viewmodel_list_user_assigned = new List<MachineListItemViewModel>();
            List<MachineListItemViewModel> viewmodel_list_not_user_assigned = new List<MachineListItemViewModel>();
            foreach (Machine machine in machine_list)
            {
                if(!((InUseInterface_Proxy)machine.Inuse).IsNull)
                {
                    MachineListItemViewModel new_viewmodel = new MachineListItemViewModel(_NavigationService, _PageDialogService)
                    {
                        IsInUseByMe = true,
                    };
                    await new_viewmodel.LoadInstance(machine).ConfigureAwait(false);
                    viewmodel_list_user_assigned.Add(new_viewmodel);
                }
                else
                {
                    MachineListItemViewModel new_viewmodel = new MachineListItemViewModel(_NavigationService, _PageDialogService);
                    await new_viewmodel.LoadInstance(machine).ConfigureAwait(false);
                    viewmodel_list_not_user_assigned.Add(new_viewmodel);
                }
            }

            List<MachineListItemViewModel> viewmodel_list_sorted = new List<MachineListItemViewModel>();
            viewmodel_list_sorted.AddRange(viewmodel_list_user_assigned.OrderBy(x => x.Machine.Name, StringComparison.OrdinalIgnoreCase.WithNaturalSort()));
            viewmodel_list_sorted.AddRange(viewmodel_list_not_user_assigned.OrderBy(x => x.Machine.Name, StringComparison.OrdinalIgnoreCase.WithNaturalSort()));
            MachineListItemViewModel_List = viewmodel_list_sorted;
        }
        #endregion

        #region Fields
        private IList<MachineListItemViewModel> _MachineListItemViewModel_List;
        public IList<MachineListItemViewModel> MachineListItemViewModel_List
        {
            get => _MachineListItemViewModel_List;
            set => SetProperty(ref _MachineListItemViewModel_List, value);
        }

        private bool _IsRefreshing;
        public bool IsRefreshing
        {
            get => _IsRefreshing;
            set => SetProperty(ref _IsRefreshing, value);
        }
        #endregion

        #region Commands
        private ICommand _RefreshCommand;
        public ICommand RefreshCommand
        {
            get => _RefreshCommand;
            set => SetProperty(ref _RefreshCommand, value);
        }
        public async Task RefreshCommandExecute()
        {
            if(_API.IsConnected)
            {
                await LoadAPIData().ConfigureAwait(true);
            }
            
            IsRefreshing = false;
        }

        private ICommand _ScanCodeCommand;
        public ICommand ScanCodeCommand
        {
            get => _ScanCodeCommand;
            set => SetProperty(ref _ScanCodeCommand, value);
        }
        public async Task ScanCodeCommandExecute()
        {
            IsBusy = true;

            NavigationParameters parameters = new NavigationParameters()
            {
                {"intance", null },
            };

            await _NavigationService.NavigateAsync("ScanURNPage", parameters).ConfigureAwait(false);
        }
        #endregion
    }
}
